﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    public abstract class PopExpression<T> : StatelessExpression
    {
        public void Eval(IStack<T> stack)
        {
            if (stack.Count == 0)
                return;

            stack.Pop();
        }
    }

    public class IntegerPopExpression : PopExpression<long>
    {
        protected override string InitStringRepresentation() { return "INTEGER.POP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.IntegerStack);
        }
    }

    public class FloatPopExpression : PopExpression<double>
    {
        protected override string InitStringRepresentation() { return "FLOAT.POP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.FloatStack);
        }
    }

    public class BooleanPopExpression : PopExpression<bool>
    {
        protected override string InitStringRepresentation() { return "BOOLEAN.POP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.BooleanStack);
        }
    }

    public class NamePopExpression : PopExpression<string>
    {
        protected override string InitStringRepresentation() { return "NAME.POP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.NameStack);
        }
    }

    public class ExecPopExpression : PopExpression<Expression>
    {
        protected override string InitStringRepresentation() { return "EXEC.POP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.ExecStack);
        }
    }

    public class CodePopExpression : PopExpression<Expression>
    {
        protected override string InitStringRepresentation() { return "CODE.POP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.CodeStack);
        }
    }
}
